@@ -12,10 +12,13 @@
 #include "js/TypeDecls.h"
 
 // Forward declarations of all the types a TraceKind can denote.
+class JSLinearString;
+
 namespace js {
 class BaseScript;
 class BaseShape;
-class ObjectGroup;
+class GetterSetter;
+class PropMap;
 class RegExpShared;
 class Shape;
 class Scope;
@@ -39,6 +42,7 @@ enum class TraceKind {
   // These trace kinds have a publicly exposed, although opaque, C++ type.
   // Note: The order here is determined by our Value packing. Other users
   //       should sort alphabetically, for consistency.
+  // Note: Nursery allocatable kinds go first. See js::gc::NurseryTraceKinds.
   Object = 0x00,
   BigInt = 0x01,
   String = 0x02,
@@ -47,32 +51,25 @@ enum class TraceKind {
   // Shape details are exposed through JS_TraceShapeCycleCollectorChildren.
   Shape = 0x04,
 
-  // ObjectGroup details are exposed through
-  // JS_TraceObjectGroupCycleCollectorChildren.
-  ObjectGroup = 0x05,
+  BaseShape = 0x05,
 
   // The kind associated with a nullptr.
   Null = 0x06,
 
   // The following kinds do not have an exposed C++ idiom.
-  BaseShape = 0x0F,
-  JitCode = 0x1F,
-  Script = 0x2F,
-  Scope = 0x3F,
-  RegExpShared = 0x4F
+  JitCode,
+  Script,
+  Scope,
+  RegExpShared,
+  GetterSetter,
+  PropMap,
 };
-const static uintptr_t OutOfLineTraceKindMask = 0x07;
 
-#define ASSERT_TRACE_KIND(tk)                                             \
-  static_assert(                                                          \
-      (uintptr_t(tk) & OutOfLineTraceKindMask) == OutOfLineTraceKindMask, \
-      "mask bits are set")
-ASSERT_TRACE_KIND(JS::TraceKind::BaseShape);
-ASSERT_TRACE_KIND(JS::TraceKind::JitCode);
-ASSERT_TRACE_KIND(JS::TraceKind::Script);
-ASSERT_TRACE_KIND(JS::TraceKind::Scope);
-ASSERT_TRACE_KIND(JS::TraceKind::RegExpShared);
-#undef ASSERT_TRACE_KIND
+// GCCellPtr packs the trace kind into the low bits of the pointer for common
+// kinds.
+const static uintptr_t OutOfLineTraceKindMask = 0x07;
+static_assert(uintptr_t(JS::TraceKind::Null) < OutOfLineTraceKindMask,
+              "GCCellPtr requires an inline representation for nullptr");
 
 // When this header is imported inside SpiderMonkey, the class definitions are
 // available and we can query those definitions to find the correct kind
@@ -97,13 +94,14 @@ struct MapTypeToTraceKind {
   D(JitCode,      js::jit::JitCode, true,           false)       \
   D(Scope,        js::Scope,        true,           true)        \
   D(Object,       JSObject,         true,           true)        \
-  D(ObjectGroup,  js::ObjectGroup,  true,           false)       \
   D(Script,       js::BaseScript,   true,           true)        \
   D(Shape,        js::Shape,        true,           false)       \
   D(String,       JSString,         false,          false)       \
   D(Symbol,       JS::Symbol,       false,          false)       \
   D(BigInt,       JS::BigInt,       false,          false)       \
-  D(RegExpShared, js::RegExpShared, true,           true)
+  D(RegExpShared, js::RegExpShared, true,           true)        \
+  D(GetterSetter, js::GetterSetter, true,           true)        \
+  D(PropMap,      js::PropMap,      false,          false)
 // clang-format on
 
 // Returns true if the JS::TraceKind is represented as a node in cycle collector
@@ -144,6 +142,19 @@ inline constexpr bool IsBaseTraceType_v = IsBaseTraceType<T>::value;
 JS_FOR_EACH_TRACEKIND(JS_EXPAND_DEF);
 #undef JS_EXPAND_DEF
 
+template <>
+struct MapTypeToTraceKind<JSLinearString> {
+  static const JS::TraceKind kind = JS::TraceKind::String;
+};
+template <>
+struct MapTypeToTraceKind<JSFunction> {
+  static const JS::TraceKind kind = JS::TraceKind::Object;
+};
+template <>
+struct MapTypeToTraceKind<JSScript> {
+  static const JS::TraceKind kind = JS::TraceKind::Script;
+};
+
 // RootKind is closely related to TraceKind. Whereas TraceKind's indices are
 // laid out for convenient embedding as a pointer tag, the indicies of RootKind
 // are designed for use as array keys via EnumeratedArray.
@@ -203,11 +214,6 @@ template <>
 struct MapTypeToRootKind<jsid> {
   static const JS::RootKind kind = JS::RootKind::Id;
 };
-template <>
-struct MapTypeToRootKind<JSFunction*> : public MapTypeToRootKind<JSObject*> {};
-template <>
-struct MapTypeToRootKind<JSScript*>
-    : public MapTypeToRootKind<js::BaseScript*> {};
 
 // Fortunately, few places in the system need to deal with fully abstract
 // cells. In those places that do, we generally want to move to a layout
