@@ -5,7 +5,7 @@
 
 //-----------------------------------------------------------------------------
 var BUGNUMBER = 352742;
-var summary = 'Array filter on {valueOf: Function}';
+var summary = 'eval("return") in toString';
 var actual = '';
 var expect = '';
 
@@ -19,18 +19,11 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
 
-  print('If the test harness fails, this test fails.');
-  expect = 4; 
-  z = {valueOf: Function};
+  expect = 3;
+  var j = ({toString: function() { eval("return"); }});
   actual = 2;
-  try {
-    [11].filter(z);
-  }
-  catch(e)
-  {
-    actual = 3;
-  }
-  actual = 4;
+  try { "" + j; } catch(e){}
+  actual = 3;
 
   reportCompare(expect, actual, summary);
 }
