@@ -2,16 +2,12 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.2-2-20
+es5id: 15.2.3.2-2-8
 description: >
     Object.getPrototypeOf returns the [[Prototype]] of its parameter
-    (Function Object)
+    (Math)
 ---*/
 
-var obj = function(a, b) {
-  return a + b;
-};
-
-assert.sameValue(Object.getPrototypeOf(obj), Function.prototype, 'Object.getPrototypeOf(obj)');
+assert.sameValue(Object.getPrototypeOf(Math), Object.prototype, 'Object.getPrototypeOf(Math)');
 
 reportCompare(0, 0);
