@@ -9,11 +9,11 @@
 #include "mozilla/MemoryReporting.h"
 #include "mozilla/PodOperations.h"
 
-#include <algorithm>
 #include <type_traits>
 
 #include "builtin/RegExp.h"
 #include "builtin/SelfHostingDefines.h"  // REGEXP_*_FLAG
+#include "frontend/FrontendContext.h"    // AutoReportFrontendContext
 #include "frontend/TokenStream.h"
 #include "gc/HashUtil.h"
 #include "irregexp/RegExpAPI.h"
@@ -22,19 +22,15 @@
 #include "js/Object.h"                // JS::GetBuiltinClass
 #include "js/RegExp.h"
 #include "js/RegExpFlags.h"  // JS::RegExpFlags
-#include "js/StableStringChars.h"
 #include "util/StringBuffer.h"
 #include "vm/MatchPairs.h"
+#include "vm/PlainObject.h"
 #include "vm/RegExpStatics.h"
 #include "vm/StringType.h"
-#include "vm/TraceLogging.h"
-#ifdef DEBUG
-#  include "util/Unicode.h"
-#endif
 #include "vm/WellKnownAtom.h"  // js_*_str
 
+#include "vm/JSContext-inl.h"
 #include "vm/JSObject-inl.h"
-#include "vm/NativeObject-inl.h"
 #include "vm/Shape-inl.h"
 
 using namespace js;
@@ -153,13 +149,28 @@ bool RegExpObject::isOriginalFlagGetter(JSNative native, RegExpFlags* mask) {
   return false;
 }
 
+static bool FinishRegExpClassInit(JSContext* cx, JS::HandleObject ctor,
+                                  JS::HandleObject proto) {
+#ifdef DEBUG
+  // Assert RegExp.prototype.exec is usually stored in a dynamic slot. The
+  // optimization in InlinableNativeIRGenerator::tryAttachIntrinsicRegExpExec
+  // depends on this.
+  Handle<NativeObject*> nproto = proto.as<NativeObject>();
+  auto prop = nproto->lookupPure(cx->names().exec);
+  MOZ_ASSERT(prop->isDataProperty());
+  MOZ_ASSERT(!nproto->isFixedSlot(prop->slot()));
+#endif
+  return true;
+}
+
 static const ClassSpec RegExpObjectClassSpec = {
     GenericCreateConstructor<js::regexp_construct, 2, gc::AllocKind::FUNCTION>,
     GenericCreatePrototype<RegExpObject>,
     nullptr,
     js::regexp_static_props,
     js::regexp_methods,
-    js::regexp_properties};
+    js::regexp_properties,
+    FinishRegExpClassInit};
 
 const JSClass RegExpObject::class_ = {
     js_RegExp_str,
@@ -178,7 +189,7 @@ RegExpObject* RegExpObject::create(JSContext* cx, const CharT* chars,
   static_assert(std::is_same_v<CharT, char16_t>,
                 "this code may need updating if/when CharT encodes UTF-8");
 
-  RootedAtom source(cx, AtomizeChars(cx, chars, length));
+  Rooted<JSAtom*> source(cx, AtomizeChars(cx, chars, length));
   if (!source) {
     return nullptr;
   }
@@ -192,7 +203,7 @@ template RegExpObject* RegExpObject::create(JSContext* cx,
                                             NewObjectKind newKind);
 
 RegExpObject* RegExpObject::createSyntaxChecked(JSContext* cx,
-                                                HandleAtom source,
+                                                Handle<JSAtom*> source,
                                                 RegExpFlags flags,
                                                 NewObjectKind newKind) {
   Rooted<RegExpObject*> regexp(cx, RegExpAlloc(cx, newKind));
@@ -205,25 +216,29 @@ RegExpObject* RegExpObject::createSyntaxChecked(JSContext* cx,
   return regexp;
 }
 
-RegExpObject* RegExpObject::create(JSContext* cx, HandleAtom source,
+RegExpObject* RegExpObject::create(JSContext* cx, Handle<JSAtom*> source,
                                    RegExpFlags flags, NewObjectKind newKind) {
-  CompileOptions dummyOptions(cx);
-  frontend::DummyTokenStream dummyTokenStream(cx, dummyOptions);
+  Rooted<RegExpObject*> regexp(cx);
+  {
+    AutoReportFrontendContext fc(cx);
+    CompileOptions dummyOptions(cx);
+    frontend::DummyTokenStream dummyTokenStream(&fc, dummyOptions);
 
-  LifoAllocScope allocScope(&cx->tempLifoAlloc());
-  if (!irregexp::CheckPatternSyntax(cx, dummyTokenStream, source, flags)) {
-    return nullptr;
+    LifoAllocScope allocScope(&cx->tempLifoAlloc());
+    if (!irregexp::CheckPatternSyntax(cx, cx->stackLimitForCurrentPrincipal(),
+                                      dummyTokenStream, source, flags)) {
+      return nullptr;
+    }
+
+    regexp = RegExpAlloc(cx, newKind);
+    if (!regexp) {
+      return nullptr;
+    }
+
+    regexp->initAndZeroLastIndex(source, flags, cx);
+
+    MOZ_ASSERT(!regexp->hasShared());
   }
-
-  Rooted<RegExpObject*> regexp(cx, RegExpAlloc(cx, newKind));
-  if (!regexp) {
-    return nullptr;
-  }
-
-  regexp->initAndZeroLastIndex(source, flags, cx);
-
-  MOZ_ASSERT(!regexp->hasShared());
-
   return regexp;
 }
 
@@ -231,7 +246,7 @@ RegExpObject* RegExpObject::create(JSContext* cx, HandleAtom source,
 RegExpShared* RegExpObject::createShared(JSContext* cx,
                                          Handle<RegExpObject*> regexp) {
   MOZ_ASSERT(!regexp->hasShared());
-  RootedAtom source(cx, regexp->getSource());
+  Rooted<JSAtom*> source(cx, regexp->getSource());
   RegExpShared* shared =
       cx->zone()->regExps().get(cx, source, regexp->getFlags());
   if (!shared) {
@@ -245,8 +260,8 @@ RegExpShared* RegExpObject::createShared(JSContext* cx,
   return shared;
 }
 
-Shape* RegExpObject::assignInitialShape(JSContext* cx,
-                                        Handle<RegExpObject*> self) {
+SharedShape* RegExpObject::assignInitialShape(JSContext* cx,
+                                              Handle<RegExpObject*> self) {
   MOZ_ASSERT(self->empty());
 
   static_assert(LAST_INDEX_SLOT == 0);
@@ -258,7 +273,7 @@ Shape* RegExpObject::assignInitialShape(JSContext* cx,
     return nullptr;
   }
 
-  return self->shape();
+  return self->sharedShape();
 }
 
 void RegExpObject::initIgnoringLastIndex(JSAtom* source, RegExpFlags flags) {
@@ -415,7 +430,7 @@ static bool EscapeRegExpPattern(StringBuffer& sb, const CharT* oldChars,
 }
 
 // ES6 draft rev32 21.2.3.2.4.
-JSLinearString* js::EscapeRegExpPattern(JSContext* cx, HandleAtom src) {
+JSLinearString* js::EscapeRegExpPattern(JSContext* cx, Handle<JSAtom*> src) {
   // Step 2.
   if (src->length() == 0) {
     return cx->names().emptyRegExp;
@@ -423,32 +438,36 @@ JSLinearString* js::EscapeRegExpPattern(JSContext* cx, HandleAtom src) {
 
   // We may never need to use |sb|. Start using it lazily.
   JSStringBuilder sb(cx);
-
+  bool escapeFailed = false;
   if (src->hasLatin1Chars()) {
     JS::AutoCheckCannotGC nogc;
-    if (!::EscapeRegExpPattern(sb, src->latin1Chars(nogc), src->length())) {
-      return nullptr;
-    }
+    escapeFailed =
+        !::EscapeRegExpPattern(sb, src->latin1Chars(nogc), src->length());
   } else {
     JS::AutoCheckCannotGC nogc;
-    if (!::EscapeRegExpPattern(sb, src->twoByteChars(nogc), src->length())) {
-      return nullptr;
-    }
+    escapeFailed =
+        !::EscapeRegExpPattern(sb, src->twoByteChars(nogc), src->length());
+  }
+  if (escapeFailed) {
+    return nullptr;
   }
 
   // Step 3.
-  return sb.empty() ? src : sb.finishString();
+  if (sb.empty()) {
+    return src;
+  }
+  return sb.finishString();
 }
 
 // ES6 draft rev32 21.2.5.14. Optimized for RegExpObject.
 JSLinearString* RegExpObject::toString(JSContext* cx,
                                        Handle<RegExpObject*> obj) {
   // Steps 3-4.
-  RootedAtom src(cx, obj->getSource());
+  Rooted<JSAtom*> src(cx, obj->getSource());
   if (!src) {
     return nullptr;
   }
-  RootedLinearString escapedSrc(cx, EscapeRegExpPattern(cx, src));
+  Rooted<JSLinearString*> escapedSrc(cx, EscapeRegExpPattern(cx, src));
 
   // Step 7.
   JSStringBuilder sb(cx);
@@ -543,15 +562,6 @@ RegExpShared::RegExpShared(JSAtom* source, RegExpFlags flags)
     : CellWithTenuredGCPointer(source), pairCount_(0), flags(flags) {}
 
 void RegExpShared::traceChildren(JSTracer* trc) {
-  // Discard code to avoid holding onto ExecutablePools.
-  gc::GCRuntime* gc = &trc->runtime()->gc;
-  // Bug 1758095: isIncrementalGCInProgress() also returns true when called from
-  // the collector during a non-incremental GC.
-  if (trc->isMarkingTracer() && gc->isIncrementalGCInProgress() &&
-      gc->isShrinkingGC()) {
-    discardJitCode();
-  }
-
   TraceNullableCellHeaderEdge(trc, this, "RegExpShared source");
   if (kind() == RegExpShared::Kind::Atom) {
     TraceNullableEdge(trc, &patternAtom_, "RegExpShared pattern atom");
@@ -590,7 +600,7 @@ void RegExpShared::finalize(JS::GCContext* gcx) {
 /* static */
 bool RegExpShared::compileIfNecessary(JSContext* cx,
                                       MutableHandleRegExpShared re,
-                                      HandleLinearString input,
+                                      Handle<JSLinearString*> input,
                                       RegExpShared::CodeKind codeKind) {
   if (codeKind == RegExpShared::CodeKind::Any) {
     // We start by interpreting regexps, then compile them once they are
@@ -624,8 +634,8 @@ bool RegExpShared::compileIfNecessary(JSContext* cx,
 /* static */
 RegExpRunStatus RegExpShared::execute(JSContext* cx,
                                       MutableHandleRegExpShared re,
-                                      HandleLinearString input, size_t start,
-                                      VectorMatchPairs* matches) {
+                                      Handle<JSLinearString*> input,
+                                      size_t start, VectorMatchPairs* matches) {
   MOZ_ASSERT(matches);
 
   // TODO: Add tracelogger support
@@ -718,7 +728,7 @@ RegExpRunStatus RegExpShared::execute(JSContext* cx,
   MOZ_CRASH("Unreachable");
 }
 
-void RegExpShared::useAtomMatch(HandleAtom pattern) {
+void RegExpShared::useAtomMatch(Handle<JSAtom*> pattern) {
   MOZ_ASSERT(kind() == RegExpShared::Kind::Unparsed);
   kind_ = RegExpShared::Kind::Atom;
   patternAtom_ = pattern;
@@ -733,56 +743,19 @@ void RegExpShared::useRegExpMatch(size_t pairCount) {
 }
 
 /* static */
-bool RegExpShared::initializeNamedCaptures(JSContext* cx, HandleRegExpShared re,
-                                           HandleNativeObject namedCaptures) {
+void RegExpShared::InitializeNamedCaptures(JSContext* cx, HandleRegExpShared re,
+                                           uint32_t numNamedCaptures,
+                                           Handle<PlainObject*> templateObject,
+                                           uint32_t* captureIndices) {
   MOZ_ASSERT(!re->groupsTemplate_);
   MOZ_ASSERT(!re->namedCaptureIndices_);
 
-  // The irregexp parser returns named capture information in the form
-  // of an ArrayObject, where even elements store the capture name and
-  // odd elements store the corresponding capture index. We create a
-  // template object with a property for each capture name, and store
-  // the capture indices as a heap-allocated array.
-  MOZ_ASSERT(namedCaptures->getDenseInitializedLength() % 2 == 0);
-  uint32_t numNamedCaptures = namedCaptures->getDenseInitializedLength() / 2;
-
-  // Create a plain template object.
-  RootedPlainObject templateObject(
-      cx, NewPlainObjectWithProto(cx, nullptr, TenuredObject));
-  if (!templateObject) {
-    return false;
-  }
-
-  // Initialize the properties of the template.
-  RootedId id(cx);
-  RootedValue dummyString(cx, StringValue(cx->runtime()->emptyString));
-  for (uint32_t i = 0; i < numNamedCaptures; i++) {
-    JSString* name = namedCaptures->getDenseElement(i * 2).toString();
-    id = NameToId(name->asAtom().asPropertyName());
-    if (!NativeDefineDataProperty(cx, templateObject, id, dummyString,
-                                  JSPROP_ENUMERATE)) {
-      return false;
-    }
-  }
-
-  // Allocate the capture index array.
-  uint32_t arraySize = numNamedCaptures * sizeof(uint32_t);
-  uint32_t* captureIndices = static_cast<uint32_t*>(js_malloc(arraySize));
-  if (!captureIndices) {
-    js::ReportOutOfMemory(cx);
-    return false;
-  }
-
-  // Populate the capture index array
-  for (uint32_t i = 0; i < numNamedCaptures; i++) {
-    captureIndices[i] = namedCaptures->getDenseElement(i * 2 + 1).toInt32();
-  }
-
   re->numNamedCaptures_ = numNamedCaptures;
   re->groupsTemplate_ = templateObject;
   re->namedCaptureIndices_ = captureIndices;
+
+  uint32_t arraySize = numNamedCaptures * sizeof(uint32_t);
   js::AddCellMemory(re, arraySize, MemoryUse::RegExpSharedNamedCaptureData);
-  return true;
 }
 
 void RegExpShared::tierUpTick() {
@@ -843,7 +816,7 @@ RegExpRunStatus js::ExecuteRegExpAtomRaw(RegExpShared* re,
 
 /* static */
 RegExpRunStatus RegExpShared::executeAtom(MutableHandleRegExpShared re,
-                                          HandleLinearString input,
+                                          Handle<JSLinearString*> input,
                                           size_t start,
                                           VectorMatchPairs* matches) {
   return ExecuteAtomImpl(re, input, start, matches);
@@ -881,7 +854,7 @@ ArrayObject* RegExpRealm::createMatchResultTemplateObject(
   MOZ_ASSERT(!matchResultTemplateObjects_[kind]);
 
   /* Create template array object */
-  RootedArrayObject templateObject(
+  Rooted<ArrayObject*> templateObject(
       cx,
       NewDenseUnallocatedArray(cx, RegExpObject::MaxPairCount, TenuredObject));
   if (!templateObject) {
@@ -957,20 +930,18 @@ void RegExpRealm::traceWeak(JSTracer* trc) {
                 "RegExpRealm::optimizableRegExpInstanceShape_");
 }
 
-RegExpShared* RegExpZone::get(JSContext* cx, HandleAtom source,
+RegExpShared* RegExpZone::get(JSContext* cx, Handle<JSAtom*> source,
                               RegExpFlags flags) {
   DependentAddPtr<Set> p(cx, set_, Key(source, flags));
   if (p) {
     return *p;
   }
 
-  auto shared = Allocate<RegExpShared>(cx);
+  auto* shared = cx->newCell<RegExpShared>(source, flags);
   if (!shared) {
     return nullptr;
   }
 
-  new (shared) RegExpShared(source, flags);
-
   if (!p.add(cx, set_, Key(source, flags), shared)) {
     return nullptr;
   }
@@ -1165,7 +1136,7 @@ JS_PUBLIC_API bool JS::ExecuteRegExp(JSContext* cx, HandleObject obj,
     return false;
   }
 
-  RootedLinearString input(cx, NewStringCopyN<CanGC>(cx, chars, length));
+  Rooted<JSLinearString*> input(cx, NewStringCopyN<CanGC>(cx, chars, length));
   if (!input) {
     return false;
   }
@@ -1182,7 +1153,7 @@ JS_PUBLIC_API bool JS::ExecuteRegExpNoStatics(JSContext* cx, HandleObject obj,
   AssertHeapIsIdle();
   CHECK_THREAD(cx);
 
-  RootedLinearString input(cx, NewStringCopyN<CanGC>(cx, chars, length));
+  Rooted<JSLinearString*> input(cx, NewStringCopyN<CanGC>(cx, chars, length));
   if (!input) {
     return false;
   }
@@ -1232,20 +1203,26 @@ JS_PUBLIC_API bool JS::CheckRegExpSyntax(JSContext* cx, const char16_t* chars,
   AssertHeapIsIdle();
   CHECK_THREAD(cx);
 
+  AutoReportFrontendContext fc(cx);
   CompileOptions dummyOptions(cx);
-  frontend::DummyTokenStream dummyTokenStream(cx, dummyOptions);
+  frontend::DummyTokenStream dummyTokenStream(&fc, dummyOptions);
 
   LifoAllocScope allocScope(&cx->tempLifoAlloc());
 
   mozilla::Range<const char16_t> source(chars, length);
-  bool success =
-      irregexp::CheckPatternSyntax(cx, dummyTokenStream, source, flags);
+  bool success = irregexp::CheckPatternSyntax(
+      cx->tempLifoAlloc(), cx->stackLimitForCurrentPrincipal(),
+      dummyTokenStream, source, flags);
   error.set(UndefinedValue());
   if (!success) {
+    if (!fc.convertToRuntimeErrorAndClear()) {
+      return false;
+    }
     // We can fail because of OOM or over-recursion even if the syntax is valid.
     if (cx->isThrowingOutOfMemory() || cx->isThrowingOverRecursed()) {
       return false;
     }
+
     if (!cx->getPendingException(error)) {
       return false;
     }
