@@ -2,13 +2,7 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
-from mach.decorators import (
-    CommandArgument,
-    Command,
-)
+from mach.decorators import Command, CommandArgument
 from mach.test.providers import throw2
 
 
