@@ -58,24 +58,34 @@ __wptrunner__ = {"product": "firefox",
 def get_timeout_multiplier(test_type, run_info_data, **kwargs):
     if kwargs["timeout_multiplier"] is not None:
         return kwargs["timeout_multiplier"]
+
+    multiplier = 1
+    if run_info_data["verify"]:
+        if kwargs.get("chaos_mode_flags", None) is not None:
+            multiplier = 2
+
     if test_type == "reftest":
-        if run_info_data["debug"] or run_info_data.get("asan") or run_info_data.get("tsan"):
-            return 4
+        if (run_info_data["debug"] or
+            run_info_data.get("asan") or
+            run_info_data.get("tsan")):
+            return 4 * multiplier
         else:
-            return 2
-    elif run_info_data["debug"] or run_info_data.get("asan") or run_info_data.get("tsan"):
+            return 2 * multiplier
+    elif (run_info_data["debug"] or
+          run_info_data.get("asan") or
+          run_info_data.get("tsan")):
         if run_info_data.get("ccov"):
-            return 4
+            return 4 * multiplier
         else:
-            return 3
+            return 3 * multiplier
     elif run_info_data["os"] == "android":
-        return 4
+        return 4 * multiplier
     # https://bugzilla.mozilla.org/show_bug.cgi?id=1538725
     elif run_info_data["os"] == "win" and run_info_data["processor"] == "aarch64":
-        return 4
+        return 4 * multiplier
     elif run_info_data.get("ccov"):
-        return 2
-    return 1
+        return 2 * multiplier
+    return 1 * multiplier
 
 
 def check_args(**kwargs):
@@ -103,7 +113,6 @@ def browser_kwargs(logger, test_type, run_info_data, config, **kwargs):
                                                          **kwargs),
             "leak_check": run_info_data["debug"] and (kwargs["leak_check"] is not False),
             "asan": run_info_data.get("asan"),
-            "stylo_threads": kwargs["stylo_threads"],
             "chaos_mode_flags": kwargs["chaos_mode_flags"],
             "config": config,
             "browser_channel": kwargs["browser_channel"],
@@ -130,13 +139,16 @@ def executor_kwargs(logger, test_type, test_environment, run_info_data,
     if test_type == "wdspec":
         options = {"args": []}
         if kwargs["binary"]:
-            options["binary"] = kwargs["binary"]
+            if not "webdriver_args" in executor_kwargs:
+                executor_kwargs["webdriver_args"] = []
+            executor_kwargs["webdriver_args"].extend(["--binary", kwargs["binary"]])
         if kwargs["binary_args"]:
             options["args"] = kwargs["binary_args"]
 
         if not kwargs["binary"] and kwargs["headless"] and "--headless" not in options["args"]:
             options["args"].append("--headless")
 
+        executor_kwargs["binary_args"] = options["args"]
         capabilities["moz:firefoxOptions"] = options
 
     if kwargs["certutil_binary"] is None:
@@ -184,8 +196,9 @@ def run_info_extras(**kwargs):
           "headless": kwargs.get("headless", False) or "MOZ_HEADLESS" in os.environ,
           "fission": not kwargs.get("disable_fission"),
           "sessionHistoryInParent": (not kwargs.get("disable_fission") or
-                                     get_bool_pref("fission.sessionHistoryInParent")),
-          "swgl": get_bool_pref("gfx.webrender.software")}
+                                     not get_bool_pref("fission.disableSessionHistoryInParent")),
+          "swgl": get_bool_pref("gfx.webrender.software"),
+          "editorLegacyDirectionMode": get_bool_pref_if_exists("editor.join_split_direction.compatible_with_the_other_browsers") is False}
 
     rv.update(run_info_browser_version(**kwargs))
 
@@ -207,22 +220,34 @@ def run_info_browser_version(**kwargs):
 
 
 def update_properties():
-    return (["os", "debug", "fission", "e10s", "processor", "swgl", "domstreams"],
+    return (["os", "debug", "fission", "processor", "swgl", "domstreams", "editorLegacyDirectionMode"],
             {"os": ["version"], "processor": ["bits"]})
 
 
-def get_environ(logger, binary, debug_info, stylo_threads, headless,
-                chaos_mode_flags=None):
+def log_gecko_crashes(logger, process, test, profile_dir, symbols_path, stackwalk_binary):
+    dump_dir = os.path.join(profile_dir, "minidumps")
+
+    try:
+        return bool(mozcrash.log_crashes(logger,
+                                         dump_dir,
+                                         symbols_path=symbols_path,
+                                         stackwalk_binary=stackwalk_binary,
+                                         process=process,
+                                         test=test))
+    except OSError:
+        logger.warning("Looking for crash dump files failed")
+        return False
+
+
+def get_environ(logger, binary, debug_info, headless, chaos_mode_flags=None):
     env = test_environment(xrePath=os.path.abspath(os.path.dirname(binary)),
                            debugger=debug_info is not None,
                            useLSan=True,
                            log=logger)
-
-    env["STYLO_THREADS"] = str(stylo_threads)
     # Disable window occlusion. Bug 1733955
     env["MOZ_WINDOW_OCCLUSION"] = "0"
     if chaos_mode_flags is not None:
-        env["MOZ_CHAOSMODE"] = str(chaos_mode_flags)
+        env["MOZ_CHAOSMODE"] = hex(chaos_mode_flags)
     if headless:
         env["MOZ_HEADLESS"] = "1"
     return env
@@ -247,7 +272,7 @@ class FirefoxInstanceManager:
     __metaclass__ = ABCMeta
 
     def __init__(self, logger, binary, binary_args, profile_creator, debug_info,
-                 chaos_mode_flags, headless, stylo_threads,
+                 chaos_mode_flags, headless,
                  leak_check, stackfix_dir, symbols_path, asan):
         """Object that manages starting and stopping instances of Firefox."""
         self.logger = logger
@@ -257,7 +282,6 @@ class FirefoxInstanceManager:
         self.debug_info = debug_info
         self.chaos_mode_flags = chaos_mode_flags
         self.headless = headless
-        self.stylo_threads = stylo_threads
         self.leak_check = leak_check
         self.stackfix_dir = stackfix_dir
         self.symbols_path = symbols_path
@@ -297,7 +321,7 @@ class FirefoxInstanceManager:
         marionette_port = get_free_port()
         profile.set_preferences({"marionette.port": marionette_port})
 
-        env = get_environ(self.logger, self.binary, self.debug_info, self.stylo_threads,
+        env = get_environ(self.logger, self.binary, self.debug_info,
                           self.headless, self.chaos_mode_flags)
 
         args = self.binary_args[:] if self.binary_args else []
@@ -709,7 +733,7 @@ class FirefoxBrowser(Browser):
                  symbols_path=None, stackwalk_binary=None, certutil_binary=None,
                  ca_certificate_path=None, e10s=False, disable_fission=False,
                  stackfix_dir=None, binary_args=None, timeout_multiplier=None, leak_check=False,
-                 asan=False, stylo_threads=1, chaos_mode_flags=None, config=None,
+                 asan=False, chaos_mode_flags=None, config=None,
                  browser_channel="nightly", headless=None, preload_browser=False,
                  specialpowers_path=None, debug_test=False, **kwargs):
         Browser.__init__(self, logger)
@@ -755,7 +779,6 @@ class FirefoxBrowser(Browser):
                                                      debug_info,
                                                      chaos_mode_flags,
                                                      headless,
-                                                     stylo_threads,
                                                      leak_check,
                                                      stackfix_dir,
                                                      symbols_path,
@@ -799,18 +822,12 @@ class FirefoxBrowser(Browser):
                                  "supports_devtools": True}
 
     def check_crash(self, process, test):
-        dump_dir = os.path.join(self.instance.runner.profile.profile, "minidumps")
-
-        try:
-            return bool(mozcrash.log_crashes(self.logger,
-                                             dump_dir,
-                                             symbols_path=self.symbols_path,
-                                             stackwalk_binary=self.stackwalk_binary,
-                                             process=process,
-                                             test=test))
-        except OSError:
-            self.logger.warning("Looking for crash dump files failed")
-            return False
+        return log_gecko_crashes(self.logger,
+                                 process,
+                                 test,
+                                 self.instance.runner.profile.profile,
+                                 self.symbols_path,
+                                 self.stackwalk_binary)
 
 
 class FirefoxWdSpecBrowser(WebDriverBrowser):
@@ -818,7 +835,7 @@ class FirefoxWdSpecBrowser(WebDriverBrowser):
                  extra_prefs=None, debug_info=None, symbols_path=None, stackwalk_binary=None,
                  certutil_binary=None, ca_certificate_path=None, e10s=False,
                  disable_fission=False, stackfix_dir=None, leak_check=False,
-                 asan=False, stylo_threads=1, chaos_mode_flags=None, config=None,
+                 asan=False, chaos_mode_flags=None, config=None,
                  browser_channel="nightly", headless=None, debug_test=False, **kwargs):
 
         super().__init__(logger, binary, webdriver_binary, webdriver_args)
@@ -833,7 +850,7 @@ class FirefoxWdSpecBrowser(WebDriverBrowser):
         self.leak_check = leak_check
         self.leak_report_file = None
 
-        self.env = self.get_env(binary, debug_info, stylo_threads, headless, chaos_mode_flags)
+        self.env = self.get_env(binary, debug_info, headless, chaos_mode_flags)
 
         profile_creator = ProfileCreator(logger,
                                          prefs_root,
@@ -851,11 +868,10 @@ class FirefoxWdSpecBrowser(WebDriverBrowser):
         self.profile = profile_creator.create()
         self.marionette_port = None
 
-    def get_env(self, binary, debug_info, stylo_threads, headless, chaos_mode_flags):
+    def get_env(self, binary, debug_info, headless, chaos_mode_flags):
         env = get_environ(self.logger,
                           binary,
                           debug_info,
-                          stylo_threads,
                           headless,
                           chaos_mode_flags)
         env["RUST_BACKTRACE"] = "1"
@@ -940,3 +956,11 @@ class FirefoxWdSpecBrowser(WebDriverBrowser):
         args["supports_devtools"] = False
         args["profile"] = self.profile.profile
         return cls, args
+
+    def check_crash(self, process, test):
+        return log_gecko_crashes(self.logger,
+                                 process,
+                                 test,
+                                 self.profile.profile,
+                                 self.symbols_path,
+                                 self.stackwalk_binary)
