@@ -1,7 +1,9 @@
+# mypy: ignore-errors
+
 import json
 import os
 import sys
-from io import BytesIO
+from io import BytesIO, StringIO
 from unittest import mock
 
 import pytest
@@ -114,7 +116,7 @@ def create_updater(tests, url_base="/", **kwargs):
 
 
 def create_log(entries):
-    data = BytesIO()
+    data = StringIO()
     if isinstance(entries, list):
         logger = structuredlog.StructuredLogger("expected_test")
         handler = handlers.StreamHandler(data, formatters.JSONFormatter())
@@ -125,7 +127,7 @@ def create_log(entries):
             getattr(logger, action)(**kwargs)
         logger.remove_handler(handler)
     else:
-        data.write(json.dumps(entries).encode())
+        data.write(json.dumps(entries))
     data.seek(0)
     return data
 
@@ -189,6 +191,7 @@ def test_update_1():
 
     new_manifest = updated[0][1]
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get_test(test_id).children[0].get("expected", default_run_info) == "FAIL"
 
 
@@ -227,6 +230,7 @@ def test_update_known_intermittent_1():
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get_test(test_id).children[0].get(
         "expected", default_run_info) == ["PASS", "FAIL"]
 
@@ -250,6 +254,7 @@ def test_update_known_intermittent_2():
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get_test(test_id).children[0].get(
         "expected", default_run_info) == "FAIL"
 
@@ -291,6 +296,7 @@ def test_update_existing_known_intermittent():
 
     new_manifest = updated[0][1]
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get_test(test_id).children[0].get(
         "expected", default_run_info) == ["PASS", "ERROR", "FAIL"]
 
@@ -337,6 +343,7 @@ def test_update_remove_previous_intermittent():
 
     new_manifest = updated[0][1]
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get_test(test_id).children[0].get(
         "expected", default_run_info) == ["PASS", "ERROR"]
 
@@ -372,6 +379,7 @@ def test_update_new_test_with_intermittent():
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get_test("test.htm") is None
     assert len(new_manifest.get_test(test_id).children) == 1
     assert new_manifest.get_test(test_id).children[0].get(
@@ -401,11 +409,12 @@ def test_update_expected_tie_resolution():
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get_test(test_id).children[0].get(
         "expected", default_run_info) == ["PASS", "FAIL"]
 
 
-def test_update_reorder_expected():
+def test_update_no_reorder_expected():
     tests = [("path/to/test.htm", [test_id], "testharness",
               b"""[test.htm]
   [test1]
@@ -439,11 +448,7 @@ def test_update_reorder_expected():
                                      "status": "OK"})])
 
     updated = update(tests, log_0, log_1, log_2, update_intermittent=True)
-    new_manifest = updated[0][1]
-
-    assert not new_manifest.is_empty
-    assert new_manifest.get_test(test_id).children[0].get(
-        "expected", default_run_info) == ["PASS", "FAIL"]
+    assert not updated
 
 
 def test_update_and_preserve_unchanged_expected_intermittent():
@@ -476,17 +481,71 @@ def test_update_and_preserve_unchanged_expected_intermittent():
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
 
     run_info_1 = default_run_info.copy()
     run_info_1.update({"os": "android"})
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get_test(test_id).get(
         "expected", run_info_1) == ["PASS", "FAIL"]
     assert new_manifest.get_test(test_id).get(
         "expected", default_run_info) == "PASS"
 
 
+def test_update_intermittent():
+    tests = [("path/to/test.htm", [test_id], "testharness",
+              b"""[test.htm]
+  [test1]
+    expected:
+      if os == "linux" or os == "android": [PASS, FAIL, ERROR]""")]
+
+    # Logs where the test requires an update, but we don't want to update the
+    # intermittent status
+    log_0 = suite_log([("test_start", {"test": test_id}),
+                       ("test_status", {"test": test_id,
+                                        "subtest": "test1",
+                                        "status": "FAIL",
+                                        "expected": "PASS",
+                                        "known_intermittent": ["FAIL", "ERROR"]}),
+                       ("test_end", {"test": test_id,
+                                     "status": "OK"})])
+
+    log_1 = suite_log([("test_start", {"test": test_id}),
+                       ("test_status", {"test": test_id,
+                                        "subtest": "test1",
+                                        "status": "FAIL",
+                                        "expected": "PASS",
+                                        "known_intermittent": ["FAIL", "ERROR"]}),
+                       ("test_end", {"test": test_id,
+                                     "status": "OK"})])
+
+    log_2 = suite_log([("test_start", {"test": test_id}),
+                       ("test_status", {"test": test_id,
+                                        "subtest": "test1",
+                                        "status": "PASS",
+                                        "expected": "PASS",
+                                        "known_intermittent": ["FAIL", "ERROR"]}),
+                       ("test_end", {"test": test_id,
+                                     "status": "TIMEOUT"})],
+                      run_info={"os": "android"})
+
+    updated = update(tests, log_0, log_1, log_2, update_intermittent=True)
+    new_manifest = updated[0][1]
+
+    assert not new_manifest.is_empty
+    assert new_manifest.modified
+
+    run_info_1 = default_run_info.copy()
+    run_info_1.update({"os": "android"})
+    assert new_manifest.get_test(test_id).get(
+        "expected", run_info_1) == "TIMEOUT"
+
+    assert new_manifest.get_test(test_id).children[0].get(
+        "expected", default_run_info) == ["PASS", "FAIL", "ERROR"]
+
+
 def test_update_test_with_intermittent_to_one_expected_status():
     tests = [("path/to/test.htm", [test_id], "testharness",
               b"""[test.htm]
@@ -506,6 +565,7 @@ def test_update_test_with_intermittent_to_one_expected_status():
 
     new_manifest = updated[0][1]
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get_test(test_id).children[0].get(
         "expected", default_run_info) == "ERROR"
 
@@ -534,11 +594,13 @@ def test_update_intermittent_with_conditions():
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
 
     run_info_1 = default_run_info.copy()
     run_info_1.update({"os": "android"})
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get_test(test_id).get(
         "expected", run_info_1) == ["PASS", "TIMEOUT", "FAIL"]
 
@@ -567,11 +629,13 @@ def test_update_and_remove_intermittent_with_conditions():
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
 
     run_info_1 = default_run_info.copy()
     run_info_1.update({"os": "android"})
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get_test(test_id).get(
         "expected", run_info_1) == ["PASS", "TIMEOUT"]
 
@@ -606,6 +670,7 @@ def test_update_intermittent_full():
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     run_info_1 = default_run_info.copy()
     run_info_1.update({"os": "mac"})
     assert new_manifest.get_test(test_id).children[0].get(
@@ -655,8 +720,10 @@ def test_update_intermittent_full_remove():
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     run_info_1 = default_run_info.copy()
     run_info_1.update({"os": "mac"})
+    assert new_manifest.modified
     assert new_manifest.get_test(test_id).children[0].get(
         "expected", run_info_1) == ["FAIL", "TIMEOUT"]
     assert new_manifest.get_test(test_id).children[0].get(
@@ -693,8 +760,10 @@ def test_full_update():
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     run_info_1 = default_run_info.copy()
     run_info_1.update({"os": "mac"})
+    assert new_manifest.modified
     assert new_manifest.get_test(test_id).children[0].get(
         "expected", run_info_1) == "FAIL"
     assert new_manifest.get_test(test_id).children[0].get(
@@ -726,13 +795,14 @@ def test_full_orphan():
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert len(new_manifest.get_test(test_id).children[0].children) == 0
     assert new_manifest.get_test(test_id).children[0].get(
         "expected", default_run_info) == "FAIL"
     assert len(new_manifest.get_test(test_id).children) == 1
 
 
-def test_update_reorder_expected_full_conditions():
+def test_update_no_reorder_expected_full_conditions():
     tests = [("path/to/test.htm", [test_id], "testharness",
               b"""[test.htm]
   [test1]
@@ -779,16 +849,7 @@ def test_update_reorder_expected_full_conditions():
                                    "status": "OK"})])
 
     updated = update(tests, log_0, log_1, log_2, log_3, update_intermittent=True, full_update=True)
-
-    new_manifest = updated[0][1]
-
-    assert not new_manifest.is_empty
-    run_info_1 = default_run_info.copy()
-    run_info_1.update({"os": "mac"})
-    assert new_manifest.get_test(test_id).children[0].get(
-        "expected", run_info_1) == ["TIMEOUT", "FAIL"]
-    assert new_manifest.get_test(test_id).children[0].get(
-        "expected", default_run_info) == ["PASS", "FAIL"]
+    assert not updated
 
 
 def test_skip_0():
@@ -828,10 +889,38 @@ def test_new_subtest():
     updated = update(tests, log)
     new_manifest = updated[0][1]
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get_test(test_id).children[0].get("expected", default_run_info) == "FAIL"
     assert new_manifest.get_test(test_id).children[1].get("expected", default_run_info) == "FAIL"
 
 
+def test_update_subtest():
+    tests = [("path/to/test.htm", [test_id], "testharness", b"""[test.htm]
+  expected:
+    if os == "linux": [OK, ERROR]
+  [test1]
+    expected: FAIL""")]
+
+    log = suite_log([("test_start", {"test": test_id}),
+                     ("test_status", {"test": test_id,
+                                      "subtest": "test1",
+                                      "status": "FAIL",
+                                      "known_intermittent": []}),
+                     ("test_status", {"test": test_id,
+                                      "subtest": "test2",
+                                      "status": "FAIL",
+                                      "expected": "PASS",
+                                      "known_intermittent": []}),
+                     ("test_end", {"test": test_id,
+                                   "status": "OK",
+                                   "known_intermittent": ["ERROR"]})])
+    updated = update(tests, log)
+    new_manifest = updated[0][1]
+    assert not new_manifest.is_empty
+    assert new_manifest.modified
+    assert new_manifest.get_test(test_id).children[0].get("expected", default_run_info) == "FAIL"
+
+
 def test_update_multiple_0():
     tests = [("path/to/test.htm", [test_id], "testharness", b"""[test.htm]
   [test1]
@@ -859,6 +948,7 @@ def test_update_multiple_0():
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     run_info_1 = default_run_info.copy()
     run_info_1.update({"debug": False, "os": "osx"})
     run_info_2 = default_run_info.copy()
@@ -896,6 +986,7 @@ def test_update_multiple_1():
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     run_info_1 = default_run_info.copy()
     run_info_1.update({"os": "osx"})
     run_info_2 = default_run_info.copy()
@@ -943,6 +1034,7 @@ def test_update_multiple_2():
     run_info_2.update({"debug": True, "os": "osx"})
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get_test(test_id).children[0].get(
         "expected", run_info_1) == "FAIL"
     assert new_manifest.get_test(test_id).children[0].get(
@@ -983,6 +1075,7 @@ def test_update_multiple_3():
     run_info_2.update({"debug": True, "os": "osx"})
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get_test(test_id).children[0].get(
         "expected", run_info_1) == "FAIL"
     assert new_manifest.get_test(test_id).children[0].get(
@@ -1023,6 +1116,7 @@ def test_update_ignore_existing():
     run_info_2.update({"debug": False, "os": "osx"})
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get_test(test_id).children[0].get(
         "expected", run_info_1) == "FAIL"
     assert new_manifest.get_test(test_id).children[0].get(
@@ -1045,6 +1139,7 @@ def test_update_new_test():
     run_info_1 = default_run_info.copy()
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get_test("test.htm") is None
     assert len(new_manifest.get_test(test_id).children) == 1
     assert new_manifest.get_test(test_id).children[0].get(
@@ -1167,6 +1262,7 @@ def test_update_full():
     run_info_2.update({"debug": True, "os": "osx"})
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get_test("test.js") is None
     assert len(new_manifest.get_test(test_id).children) == 1
     assert new_manifest.get_test(test_id).children[0].get(
@@ -1210,6 +1306,7 @@ def test_update_full_unknown():
     run_info_2.update({"release_or_beta": True})
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get_test(test_id).children[0].get(
         "expected", run_info_1) == "FAIL"
     assert new_manifest.get_test(test_id).children[0].get(
@@ -1266,6 +1363,7 @@ def test_update_default():
     new_manifest = updated[0][1]
 
     assert new_manifest.is_empty
+    assert new_manifest.modified
 
 
 def test_update_default_1():
@@ -1285,6 +1383,7 @@ def test_update_default_1():
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
 
     run_info_1 = default_run_info.copy()
     run_info_1.update({"os": "mac"})
@@ -1292,6 +1391,7 @@ def test_update_default_1():
     run_info_2.update({"os": "win"})
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get_test(test_id).get(
         "expected", run_info_1) == "TIMEOUT"
     assert new_manifest.get_test(test_id).get(
@@ -1315,6 +1415,7 @@ def test_update_default_2():
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
 
     run_info_1 = default_run_info.copy()
     run_info_1.update({"os": "mac"})
@@ -1322,6 +1423,7 @@ def test_update_default_2():
     run_info_2.update({"os": "win"})
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get_test(test_id).get(
         "expected", run_info_1) == "TIMEOUT"
     assert new_manifest.get_test(test_id).get(
@@ -1346,6 +1448,7 @@ def test_update_assertion_count_0():
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get_test(test_id).get("max-asserts") == "7"
     assert new_manifest.get_test(test_id).get("min-asserts") == "2"
 
@@ -1368,6 +1471,7 @@ def test_update_assertion_count_1():
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get_test(test_id).get("max-asserts") == "4"
     assert new_manifest.get_test(test_id).has_key("min-asserts") is False
 
@@ -1418,6 +1522,7 @@ def test_update_assertion_count_3():
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get_test(test_id).get("max-asserts") == "8"
     assert new_manifest.get_test(test_id).get("min-asserts") == "2"
 
@@ -1447,6 +1552,7 @@ def test_update_assertion_count_4():
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get_test(test_id).get("max-asserts") == "8"
     assert new_manifest.get_test(test_id).has_key("min-asserts") is False
 
@@ -1463,6 +1569,7 @@ def test_update_lsan_0():
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get("lsan-allowed") == ["foo"]
 
 
@@ -1481,6 +1588,7 @@ lsan-allowed: [foo]""")]
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get("lsan-allowed") == ["baz", "foo"]
 
 
@@ -1501,6 +1609,7 @@ lsan-allowed: [foo]"""),
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get("lsan-allowed") == ["baz"]
 
 
@@ -1521,6 +1630,7 @@ def test_update_lsan_3():
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get("lsan-allowed") == ["baz", "foo"]
 
 
@@ -1573,6 +1683,7 @@ def test_update_leak_total_0():
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get("leak-threshold") == ['default:51200']
 
 
@@ -1620,6 +1731,7 @@ leak-total: 100""")]
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.get("leak-threshold") == ['default:51200']
 
 
@@ -1641,10 +1753,13 @@ leak-total: 110""")]
     new_manifest = updated[0][1]
 
     assert not new_manifest.is_empty
+    assert new_manifest.modified
     assert new_manifest.has_key("leak-threshold") is False
 
 
 class TestStep(Step):
+    __test__ = False
+
     def create(self, state):
         tests = [("path/to/test.htm", [test_id], "testharness", "")]
         state.foo = create_test_manifest(tests)
